# github-version-checker
> Simple **version checker** working with **GitHub releases** and the **GitHub API**.

# Note on upgrading to **v2.0.0**
If you are upgrading from a version before **v2.0.0**, please read [**this guide**](MIGRATING-TO-2.0.0.md).

## Install
```bash
$ npm install github-version-checker
```

## Usage
```javascript
// require modules
const versionCheck = require('github-version-checker');
const pkg = require('./package.json'); // or whereever your package.json lies

// version check options (for details see below)
const options = {
  token: 'PUT-YOUR-TOKEN-HERE',                      // personal access token
  repo: 'github-version-checker',                    // repository name
  owner: 'axelrindle',                               // repository owner
  currentVersion: pkg.version,                       // your app's current version
  fetchTags: false                                   // whether to fetch releases or tags
};
versionCheck(options, function (error, update) { // callback function
  if (error) throw error;
  if (update) { // print some update info if an update is available
    console.log('An update is available! ' + update.name);
  }

  // start your app
  console.log('Starting app...');
  //...
});
```
Check the [examples folder](https://github.com/axelrindle/github-version-checker/tree/master/examples) for more examples.

### Note on semantic versioning
Always try to follow a clear [semver format](http://semver.org/). This helps to avoid parsing mistakes and getting a clear result. Also read [here](https://github.com/npm/node-semver#versions).

## API
### `function versionCheck(options, [callback])`
Performs an update check with the given options. The `callback` is optional, can be left out to return a `Promise`.

#### The `options` object
Option | Description | Default Value
--- | --- | ---
token |  A [personal access token](https://blog.github.com/2013-05-16-personal-api-tokens/) used to access the Github GraphQL API (as of version **2.0.0**).
repo | The name of your Github repository.
owner | The owner of your Github repository (usually your username).
currentVersion | Your app's current version. Needed to check if a newer release is found.
fetchTags | Whether to fetch the repositorie's git tags instead of the GitHub releases (this is useful when the releases are autogenerated from the tags). | `false`

#### The `callback` function (optional)
Should be of the following form:
```javascript
function(error, update) {
  // ...your code
}
```
* `error`:
  * If an error occurs, this holds the error message. `null` if no error occurs.
* `update`:
  * An object in the format specified below. `null` if no update was found.

#### Using `Promise`
You can omit the `callback` function to return a `Promise`, which resolved with the `update` object.

### Object schemes
#### Releases
When fetching releases, an object with the following structure will be returned:
```js
Object {
  name
  tag {
    name
  }
  isPrerelease
  publishedAt
  url
}
```

#### Tags
When fetching tags, you will receive an object with the following structure:
```js
Object {
  name
}
```

## License
[MIT](LICENSE)
